import ipaddress
import jsonschema
from inventory_provider import juniper


def test_interface_list(netconf_doc):

    schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "name": {"type": "string"},
                "description": {"type": "string"},
                "bundle": {
                    "type": "array",
                    "items": {"type": "string"}
                },
                "ipv4": {
                    "type": "array",
                    "items": {"type": "string"}
                },
                "ipv6": {
                    "type": "array",
                    "items": {"type": "string"}
                }
            },
            "required": ["name", "description", "ipv4", "ipv6"],
            "additionalProperties": False
        }
    }

    interfaces = list(juniper.list_interfaces(netconf_doc))
    jsonschema.validate(interfaces, schema)
    assert interfaces  # at least shouldn't be empty

from lxml import etree
def test_bgp_peering_data(netconf_doc):

    active_peers = set()
    inactive_peers = set()
    for neighbor in netconf_doc.xpath('//group/neighbor'):
        address = neighbor.find('name').text
        address = ipaddress.ip_address(address).exploded
        inactive = neighbor.get('inactive')
        if inactive == 'inactive':
            inactive_peers.add(address)
        else:
            if address == '62.40.99.50':
                print('here')
            active_peers.add(address)

    peerings = list(juniper.all_bgp_peers(netconf_doc))
    jsonschema.validate(peerings, juniper.PEERING_LIST_SCHEMA)
    assert peerings  # there's always at least one

    # confirm the addresses are in canonical (exploded) form
    for p in peerings:
        canonical_address = ipaddress.ip_address(p['address']).exploded
        assert p['address'] == canonical_address

    # confirm we got all of the active peerings
    returned_addresses = {p['address'] for p in peerings}
    missing = active_peers - returned_addresses
    if missing:
        print('here')
    assert returned_addresses == active_peers

    # confirm that none of the inactive peerings were returned
    assert not inactive_peers & returned_addresses

def test_snmp_community_string(mocked_netifaces, netconf_doc):
    assert juniper.snmp_community_string(netconf_doc) == '0pBiFbD'


def test_interface_addresses_list(netconf_doc):
    schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",

        "definitions": {
            "v4a": {
                "type": "string",
                "pattern": r'^(\d+\.){3}\d+$'
            },
            "v6a": {
                "type": "string",
                "pattern": r'^([a-f\d]{4}:){7}[a-f\d]{4}$'
            },
            "v4i": {
                "type": "string",
                "pattern": r'^(\d+\.){3}\d+/\d+$'
            },
            "v6i": {
                "type": "string",
                "pattern": r'^[a-f\d:]+/\d+$'
            }
        },

        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "name": {
                    "oneOf": [
                        {"$ref": "#/definitions/v4a"},
                        {"$ref": "#/definitions/v6a"}
                    ]
                },
                "interface address": {
                    "oneOf": [
                        {"$ref": "#/definitions/v4i"},
                        {"$ref": "#/definitions/v6i"}
                    ]
                },
                "interface name": {"type": "string"},
            },
            "required": ["name", "interface address", "interface name"],
            "additionalProperties": False
        }
    }

    addresses = list(juniper.interface_addresses(netconf_doc))
    jsonschema.validate(addresses, schema)
